package protocolP2P;
import protocolP2P.Payload;
import protocolP2P.RequestResponseCode;
import exception.ProtocolError;
import exception.InternalError;

/** Representation of payload for load response.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class FilePart extends Payload {
	private static final RequestResponseCode requestResponseCode = RequestResponseCode.LOAD_RESPONSE;
	private String filename;
	private long totalSize;
	private long offset;
	private byte[] partialContent;

	/** Constructor (typically used by server) with informations about file part to send as parameters.
	 * @param filename	name of the file to send
	 * @param totalSize	total size of the file to send
	 * @param offset	where in the file begins the part we are sending
	 * @param partialContent	content of the file we send
	 * @throws	InternalError
	 */
	public FilePart(String filename, long totalSize, long offset, byte[] partialContent) throws InternalError {
		/* asserts to help debugging */
		assert totalSize >= 0 : "totalSize cannot be negative";
		assert offset >= 0 : "offset cannot be negative";
		if (totalSize < 0 || offset < 0) {
			throw new InternalError();
		}
		this.filename = filename;
		this.totalSize = totalSize;
		this.offset = offset;
		this.partialContent = partialContent;
	}

	/** Constructor (typically used by client) with datagram received as parameter.
	 * @param datagram	the full datagram received
	 * @throws	SizeError
	 * @throws	InternalError
	 */
	protected FilePart(byte[] datagram) throws SizeError, ProtocolError, InternalError {
		/* assert to help debugging */
		assert RequestResponseCode.fromCode(datagram[RequestResponseCode.RRCODE_POSITION]) == RequestResponseCode.LOAD_RESPONSE : "FilePart subclass is incompatible with this datagram, request/response code must be checked before using this constructor";
		/* InternalErrorException */
		if (RequestResponseCode.fromCode(datagram[RequestResponseCode.RRCODE_POSITION]) != RequestResponseCode.LOAD_RESPONSE) {
			throw new InternalError();
		}
		setOffset(datagram); // this can throw SizeError
		setTotalFileSize(datagram); // this can throw SizeError
		int size = getPayloadSize(datagram); // this can throw SizeError
		int partialContentStart = setFilename(datagram, size) + 1; // this can throw ProtocolError
		setPartialContent(datagram, partialContentStart, size);
	}

	/** Returns a byte[] containing datagram with padding.
	 * This datagram is still incomplete and should not be send directly.
	 * ProtocolP2PDatagram will use this method to generate the complete datagram.
	 * @return datagram with padding
	 */
	protected byte[] toDatagram() {
		//TODO : calculate payload size
		int size = ;
		byte[] datagram = new byte[size]; // java initialize all to zero
		// size is keep blank (ProtocolP2PDatagram will handle it)
		// set request/response code
		datagram[RequestResponseCode.RRCODE_POSITION] = requestResponseCode.codeValue;
		// bits 16-31 are reserved for future use
		datagram = setPayloadSize(size, datagram);
		//TODO : write offset to datagram
		//TODO : write totalFileSize to datagram
		//TODO : write filename\n to datagram
		//TODO : write partialContent to datagram
	}

	private setOffset(byte[] datagram) throws SizeError {
		//TODO: copy from 8 to 15 into long and write into offset.
		// throw protocolerror if offset is < 0
	}
	private setTotalFileSize(byte[] datagram) throw SizeError {
		// TODO: convert from byte 16 to 23 (included)
		// into long and write into totalFileSize
		// throw protocolerror if offset is < 0
	}
	private int setFilename(byte[] datagram, int payloadSize) throws ProtocolError {
		// TODO: copy datagram from byte 24 to the first \n (excluded)
		// into filename unless we excess size
		// (in this case the wrong size has been
		// set by the emitter, it is a protocolerror)
		// return position of the \n
		
	}
	private setPartialContent(byte[] datagram, int start, int payloadSize) {
		// TODO: copy datagram from start to size into partialContent
	}

}
